@@ -212,6 +212,11 @@ void *async_loop(void *arg1) {
 	uwsgi.async_runqueue = NULL;
 	uwsgi.async_runqueue_cnt = 0;
 
+	if (uwsgi.signal_socket > -1) {
+		event_queue_add_fd_read(uwsgi.async_queue, uwsgi.signal_socket);
+		event_queue_add_fd_read(uwsgi.async_queue, uwsgi.my_signal_socket);
+	}
+
 	// set a default request manager
         if (!uwsgi.schedule_to_req) uwsgi.schedule_to_req = async_schedule_to_req;
 
@@ -249,6 +254,11 @@ void *async_loop(void *arg1) {
 			// manage events
 			interesting_fd = event_queue_interesting_fd(events, i);
 
+			if (uwsgi.signal_socket > -1 && (interesting_fd == uwsgi.signal_socket || interesting_fd == uwsgi.my_signal_socket)) {
+				uwsgi_receive_signal(interesting_fd, "worker", uwsgi.mywid);
+				continue;
+			}
+
 			is_a_new_connection = 0;
 
 			// new request coming in ?
@@ -267,7 +277,7 @@ void *async_loop(void *arg1) {
 							uwsgi_log("async queue is full !!!\n");
 							last_now = now;
 						}
-						break;;
+						break;
 					}
 
 					wsgi_req_setup(uwsgi.wsgi_req, uwsgi.wsgi_req->async_id, uwsgi_sock );
@@ -285,14 +295,14 @@ void *async_loop(void *arg1) {
 
 // on linux we do not need to reset the socket to blocking state
 #ifndef __linux__
-					if (uwsgi.numproc > 1) {
-	                                	/* re-set blocking socket */
-	                                	if (fcntl(uwsgi.wsgi_req->poll.fd, F_SETFL, uwsgi_sock->arg) < 0) {
-	                                        	uwsgi_error("fcntl()");
-							uwsgi.async_queue_unused_ptr++;
-							uwsgi.async_queue_unused[uwsgi.async_queue_unused_ptr] = uwsgi.wsgi_req;
-	                                		break;
-						}
+	                                /* re-set blocking socket */
+					int arg = uwsgi_sock->arg;
+					arg &= (~O_NONBLOCK);
+	                                if (fcntl(uwsgi.wsgi_req->poll.fd, F_SETFL, arg) < 0) {
+	                                       	uwsgi_error("fcntl()");
+						uwsgi.async_queue_unused_ptr++;
+						uwsgi.async_queue_unused[uwsgi.async_queue_unused_ptr] = uwsgi.wsgi_req;
+	                                	break;
 					}
 #endif
 
