@@ -28,6 +28,14 @@ void suspend_resume_them_all(int signum) {
 		uwsgi.workers[0].suspended = 1;
 	}
 
+	// subscribe/unsubscribe if needed
+	struct uwsgi_string_list *subscriptions = uwsgi.subscriptions;
+	while(subscriptions) {
+		uwsgi_log("%s %s\n", suspend ? "unsubscribing from" : "subscribing to", subscriptions->value);
+		uwsgi_subscribe(subscriptions->value, suspend);
+		subscriptions = subscriptions->next;
+	}
+
 	for (i = 1; i <= uwsgi.numproc; i++) {
 		uwsgi.workers[i].suspended = suspend;
 		if (uwsgi.workers[i].pid > 0) {
@@ -110,6 +118,39 @@ void *logger_thread_loop(void *noarg) {
 	return NULL;
 }
 
+void *cache_sweeper_loop(void *noarg) {
+
+	int i;
+	// block all signals
+        sigset_t smask;
+        sigfillset(&smask);
+        pthread_sigmask(SIG_BLOCK, &smask, NULL);
+
+	if (!uwsgi.cache_expire_freq) uwsgi.cache_expire_freq = 3;
+
+	// remove expired cache items TODO use rb_tree timeouts
+	for(;;) {
+		sleep(uwsgi.cache_expire_freq);
+		uint64_t freed_items = 0;
+		// skip the first slot
+                for (i = 1; i < (int) uwsgi.cache_max_items; i++) {
+                	uwsgi_wlock(uwsgi.cache_lock);
+                        if (uwsgi.cache_items[i].expires) {
+                        	if (uwsgi.cache_items[i].expires < (uint64_t) uwsgi.current_time) {
+                                	uwsgi_cache_del(NULL, 0, i);
+					freed_items++;
+                                }
+                        }
+                        uwsgi_rwunlock(uwsgi.cache_lock);
+        	}
+		if (uwsgi.cache_report_freed_items && freed_items > 0) {
+			uwsgi_log("freed %llu cache items\n", (unsigned long long) freed_items);
+		}
+	};
+
+	return NULL;
+}
+
 void uwsgi_subscribe(char *subscription, uint8_t cmd) {
 
 	int subfile_size;
@@ -279,6 +320,7 @@ int master_loop(char **argv, char **environ) {
 	uint64_t last_request_count = 0;
 
 	pthread_t logger_thread;
+	pthread_t cache_sweeper;
 
 #ifdef UWSGI_UDP
 	struct sockaddr_in udp_client;
@@ -374,6 +416,16 @@ int master_loop(char **argv, char **environ) {
 		}
 	}
 
+	if (uwsgi.cache_max_items > 0 && !uwsgi.cache_no_expire) {
+		if (pthread_create(&cache_sweeper, NULL, cache_sweeper_loop, NULL)) {
+                	uwsgi_error("pthread_create()");
+			uwsgi_log("unable to run the cache sweeper !!!\n");
+                }
+		else {
+			uwsgi_log("cache sweeper thread enabled\n");
+		}
+	}
+
 
 	uwsgi.wsgi_req->buffer = uwsgi.async_buf[0];
 
@@ -683,7 +735,7 @@ int master_loop(char **argv, char **environ) {
 		}
 
 		// cheaper management
-		if (uwsgi.cheaper && !uwsgi.cheap && !uwsgi.to_heaven && !uwsgi.to_hell) {
+		if (uwsgi.cheaper && !uwsgi.cheap && !uwsgi.to_heaven && !uwsgi.to_hell && !uwsgi.workers[0].suspended) {
 			if (!uwsgi_calc_cheaper()) return 0;
 		}
 
@@ -1189,19 +1241,6 @@ int master_loop(char **argv, char **environ) {
 				}
 			}
 
-			// remove expired cache items TODO use rb_tree timeouts
-			if (uwsgi.cache_max_items > 0) {
-				for (i = 0; i < (int) uwsgi.cache_max_items; i++) {
-					uwsgi_wlock(uwsgi.cache_lock);
-					if (uwsgi.cache_items[i].expires) {
-						if (uwsgi.cache_items[i].expires < (uint64_t) uwsgi.current_time) {
-							uwsgi_cache_del(uwsgi.cache_items[i].key, uwsgi.cache_items[i].keysize);
-						}
-					}
-					uwsgi_rwunlock(uwsgi.cache_lock);
-				}
-			}
-
 			check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
 			if (!check_interval)
 				check_interval = 1;
@@ -1346,7 +1385,7 @@ int master_loop(char **argv, char **environ) {
 			}
 
 			// resubscribe every 10 cycles by default
-			if ((uwsgi.subscriptions && ((uwsgi.master_cycles % uwsgi.subscribe_freq) == 0 || uwsgi.master_cycles == 1)) && !uwsgi.to_heaven && !uwsgi.to_hell) {
+			if ((uwsgi.subscriptions && ((uwsgi.master_cycles % uwsgi.subscribe_freq) == 0 || uwsgi.master_cycles == 1)) && !uwsgi.to_heaven && !uwsgi.to_hell  && !uwsgi.workers[0].suspended) {
 				struct uwsgi_string_list *subscriptions = uwsgi.subscriptions;
 				while (subscriptions) {
 					uwsgi_subscribe(subscriptions->value, 0);
@@ -1570,7 +1609,12 @@ nextlock:
 				// noop
 			}
 			else if (uwsgi.workers[uwsgi.mywid].manage_next_request) {
-				uwsgi_log("DAMN ! worker %d (pid: %d) died :( trying respawn ...\n", uwsgi.mywid, (int) diedpid);
+				if (WIFSIGNALED(waitpid_status)) {
+					uwsgi_log("DAMN ! worker %d (pid: %d) died, killed by signal %d :( trying respawn ...\n", uwsgi.mywid, (int) diedpid, (int) WTERMSIG(waitpid_status));
+				}
+				else {
+					uwsgi_log("DAMN ! worker %d (pid: %d) died :( trying respawn ...\n", uwsgi.mywid, (int) diedpid);
+				}
 			}
 
 			if (uwsgi.workers[uwsgi.mywid].cheaped == 1) {
