@@ -8,14 +8,18 @@ void worker_wakeup() {
 int uwsgi_calc_cheaper(void) {
 
 	int i;
+	static time_t last_check = 0;
+	int check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
 
-	static time_t last_check = 0; 
- 	int check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL]; 
- 	if (!last_check) last_check = time(NULL); 
- 	time_t now = time(NULL); 
- 	if (!check_interval) check_interval = 1; 
- 	if ( (now - last_check) < check_interval) return 1; 
- 	last_check = now; 
+	if (!last_check) last_check = time(NULL);
+
+	time_t now = time(NULL);
+        if (!check_interval)
+        	check_interval = 1;
+
+	if ( (now - last_check) < check_interval) return 1;
+
+	last_check = now;
 
 	int needed_workers = uwsgi.cheaper_algo();
 
@@ -234,6 +238,9 @@ void uwsgi_reload(char **argv) {
 		waitpid(WAIT_ANY, &waitpid_status, WNOHANG);
 	}
 
+	// call atexit user exec
+	uwsgi_exec_atexit();
+
 	if (uwsgi.exit_on_reload) {
 		uwsgi_log("uWSGI: GAME OVER (insert coin)\n");
 		exit(0);
@@ -541,20 +548,36 @@ void uwsgi_manage_signal_cron(time_t now) {
 			uc_month = ucron->month;
 			uc_week = ucron->week;
 
-			if (ucron->minute == -1)
-				uc_minute = uwsgi_cron_delta->tm_min;
-			if (ucron->hour == -1)
-				uc_hour = uwsgi_cron_delta->tm_hour;
-			if (ucron->month == -1)
-				uc_month = uwsgi_cron_delta->tm_mon;
-			if (ucron->day == -1)
-				uc_day = uwsgi_cron_delta->tm_mday;
-			if (ucron->week == -1)
-				uc_week = uwsgi_cron_delta->tm_wday;
+			// negative values as interval -1 = * , -5 = */5
+			if (ucron->minute < 0) {
+				if ((uwsgi_cron_delta->tm_min % abs(ucron->minute)) == 0) {
+					uc_minute = uwsgi_cron_delta->tm_min;
+				}
+			}
+			if (ucron->hour < 0) {
+				if ((uwsgi_cron_delta->tm_hour % abs(ucron->hour)) == 0) {
+					uc_hour = uwsgi_cron_delta->tm_hour;
+				}
+			}
+			if (ucron->month < 0) {
+				if ((uwsgi_cron_delta->tm_mon % abs(ucron->month)) == 0) {
+					uc_month = uwsgi_cron_delta->tm_mon;
+				}
+			}
+			if (ucron->day < 0) {
+				if ((uwsgi_cron_delta->tm_mday % abs(ucron->day)) == 0) {
+					uc_day = uwsgi_cron_delta->tm_mday;
+				}
+			}
+			if (ucron->week < 0) {
+				if ((uwsgi_cron_delta->tm_wday % abs(ucron->week)) == 0) {	
+					uc_week = uwsgi_cron_delta->tm_wday;
+				}
+			}
 
 			int run_task = 0;
 			// mday and wday are ORed
-			if (ucron->day != -1 && ucron->week != -1) {
+			if (ucron->day >= 0 && ucron->week >= 0) {
 				if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && (uwsgi_cron_delta->tm_mday == uc_day || uwsgi_cron_delta->tm_wday == uc_week)) {
 					run_task = 1;
 				}
@@ -598,8 +621,10 @@ void uwsgi_manage_command_cron(time_t now) {
 		return;
 	}
 
-	while (current_cron) {
+	// fix month
+	uwsgi_cron_delta->tm_mon++;
 
+	while (current_cron) {
 
 		uc_minute = current_cron->minute;
 		uc_hour = current_cron->hour;
@@ -607,20 +632,36 @@ void uwsgi_manage_command_cron(time_t now) {
 		uc_month = current_cron->month;
 		uc_week = current_cron->week;
 
-		if (current_cron->minute == -1)
-			uc_minute = uwsgi_cron_delta->tm_min;
-		if (current_cron->hour == -1)
-			uc_hour = uwsgi_cron_delta->tm_hour;
-		if (current_cron->month == -1)
-			uc_month = uwsgi_cron_delta->tm_mon;
-		if (current_cron->day == -1)
-			uc_day = uwsgi_cron_delta->tm_mday;
-		if (current_cron->week == -1)
-			uc_week = uwsgi_cron_delta->tm_wday;
+		// negative values as interval -1 = * , -5 = */5
+		if (current_cron->minute < 0) {
+			if ((uwsgi_cron_delta->tm_min % abs(current_cron->minute)) == 0) {
+				uc_minute = uwsgi_cron_delta->tm_min;
+			}
+		}
+		if (current_cron->hour < 0) {
+			if ((uwsgi_cron_delta->tm_hour % abs(current_cron->hour)) == 0) {
+				uc_hour = uwsgi_cron_delta->tm_hour;
+			}
+		}
+		if (current_cron->month < 0) {
+			if ((uwsgi_cron_delta->tm_hour % abs(current_cron->month)) == 0) {
+				uc_month = uwsgi_cron_delta->tm_mon;
+			}
+		}
+		if (current_cron->day < 0) {
+			if ((uwsgi_cron_delta->tm_mday % abs(current_cron->day)) == 0) {
+				uc_day = uwsgi_cron_delta->tm_mday;
+			}
+		}
+		if (current_cron->week < 0) {
+			if ((uwsgi_cron_delta->tm_wday % abs(current_cron->week)) == 0) {
+				uc_week = uwsgi_cron_delta->tm_wday;
+			}
+		}
 
 		int run_task = 0;
 		// mday and wday are ORed
-		if (current_cron->day != -1 && current_cron->week != -1) {
+		if (current_cron->day >= 0 && current_cron->week >= 0) {
 			if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && (uwsgi_cron_delta->tm_mday == uc_day || uwsgi_cron_delta->tm_wday == uc_week)) {
 				run_task = 1;
 			}
@@ -734,6 +775,7 @@ void uwsgi_send_stats(int fd) {
 		stats_send_llu("\"delta_requests\": %llu, ", uwsgi.workers[i + 1].delta_requests);
 		stats_send_llu("\"exceptions\": %llu, ", uwsgi.workers[i + 1].exceptions);
 		stats_send_llu("\"signals\": %llu, ", uwsgi.workers[i + 1].signals);
+		stats_send_llu("\"static_offload_threads\": %llu, ", uwsgi.workers[i + 1].static_offload_threads);
 
 		if (uwsgi.workers[i + 1].cheaped) {
 			fprintf(output, "\"status\": \"cheap\", ");
