@@ -7,10 +7,6 @@ struct ufib {
 	VALUE *fib;
 } ufiber;
 
-struct option fiber_options[] = {
-	{ 0, 0, 0, 0 }
-};
-
 VALUE uwsgi_fiber_request() {
 
 	uwsgi.wsgi_req->async_status = uwsgi.p[uwsgi.wsgi_req->uh.modifier1]->request(uwsgi.wsgi_req);
@@ -58,14 +54,13 @@ void fiber_loop() {
         uwsgi.schedule_to_main = fiber_schedule_to_main;
         uwsgi.schedule_to_req = fiber_schedule_to_req;
 
-	// must run all the rack/ruby plugins without protection
 	ur.unprotected = 1;
 
         rb_protect(protected_async_loop, 0, &error);
 
         if (error) {
                 uwsgi_ruby_exception();
-                exit(1);
+		exit(1);
         }
 
         // never here
@@ -81,5 +76,4 @@ struct uwsgi_plugin fiber_plugin = {
 
 	.name = "fiber",
 	.init = fiber_init,
-	.options = fiber_options,
 };
