@@ -35,6 +35,19 @@ XS(XS_input) {
         XSRETURN(1);
 }
 
+XS(XS_psgix_logger) {
+	dXSARGS; 
+ 	psgi_check_args(1); 
+ 	HV *hv_args = (HV *) (SvRV(ST(0))); 
+ 	if (!hv_exists(hv_args, "level", 5) || !hv_exists(hv_args, "message", 7)) { 
+ 		Perl_croak(aTHX_ "psgix.logger requires bot level and message items"); 
+        } 
+ 	char *level = SvPV_nolen(*(hv_fetch(hv_args, "level", 5, 0))); 
+ 	char *message = SvPV_nolen(*(hv_fetch(hv_args, "message", 7, 0))); 
+ 	uwsgi_log("[uwsgi-perl %s] %s\n", level, message);  
+ 	XSRETURN(0); 
+}
+
 XS(XS_stream)
 {
     dXSARGS;
@@ -195,6 +208,7 @@ XS(XS_uwsgi_stacktrace) {
 
         psgi_check_args(0);
 
+	uwsgi_log("%s", SvPV_nolen(ERRSV));
 	uwsgi_log("*** uWSGI perl stacktrace ***\n");
 	SV *ret = perl_eval_pv("Devel::StackTrace->new->as_string;", 0);
         uwsgi_log("%s", SvPV_nolen(ret));
@@ -229,6 +243,7 @@ xs_init(pTHX)
         newXS("uwsgi::error::print", XS_error_print, "uwsgi::print");
         uperl.tmp_error_stash[uperl.tmp_current_i] = gv_stashpv("uwsgi::error", 0);
 
+	uperl.tmp_psgix_logger[uperl.tmp_current_i] = newXS("uwsgi::psgix_logger", XS_psgix_logger, "uwsgi");
         uperl.tmp_stream_responder[uperl.tmp_current_i] = newXS("uwsgi::stream", XS_stream, "uwsgi");
 
         newXS("uwsgi::streaming::write", XS_streaming_write, "uwsgi::streaming");
@@ -271,6 +286,7 @@ static void uwsgi_perl_free_stashes(void) {
         free(uperl.tmp_input_stash);
         free(uperl.tmp_error_stash);
         free(uperl.tmp_stream_responder);
+	free(uperl.tmp_psgix_logger); 
 }
 
 int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, PerlInterpreter **interpreters) {
@@ -334,6 +350,7 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 	uperl.tmp_input_stash = uwsgi_calloc(sizeof(HV *) * uwsgi.threads);
 	uperl.tmp_error_stash = uwsgi_calloc(sizeof(HV *) * uwsgi.threads);
 	uperl.tmp_stream_responder = uwsgi_calloc(sizeof(CV *) * uwsgi.threads);
+	uperl.tmp_psgix_logger = uwsgi_calloc(sizeof(CV *) * uwsgi.threads);
 
 	for(i=0;i<uwsgi.threads;i++) {
 
@@ -384,6 +401,7 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 
 		perl_eval_pv("use IO::Handle;", 0);
 		perl_eval_pv("use IO::File;", 0);
+		perl_eval_pv("use Scalar::Util;", 0);
 		if (!uperl.no_die_catch) {
 			perl_eval_pv("use Devel::StackTrace;", 0);
 			if (!SvTRUE(ERRSV)) {
@@ -438,6 +456,7 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 	wi->input = uperl.tmp_input_stash;
 	wi->error = uperl.tmp_error_stash;
 	wi->responder0 = uperl.tmp_stream_responder;
+	wi->responder1 = uperl.tmp_psgix_logger;
 
 	uwsgi_emulate_cow_for_apps(id);
 
