@@ -2,54 +2,9 @@
 
 extern struct uwsgi_server uwsgi;
 
-/* statistically ordered */
-struct http_status_codes hsc[] = {
-        {"200", "OK"},
-        {"302", "Found"},
-        {"404", "Not Found"},
-        {"500", "Internal Server Error"},
-        {"301", "Moved Permanently"},
-        {"304", "Not Modified"},
-        {"303", "See Other"},
-        {"403", "Forbidden"},
-        {"307", "Temporary Redirect"},
-        {"401", "Unauthorized"},
-        {"400", "Bad Request"},
-        {"405", "Method Not Allowed"},
-        {"408", "Request Timeout"},
+extern struct http_status_codes hsc[];
 
-        {"100", "Continue"},
-        {"101", "Switching Protocols"},
-        {"201", "Created"},
-        {"202", "Accepted"},
-        {"203", "Non-Authoritative Information"},
-        {"204", "No Content"},
-        {"205", "Reset Content"},
-        {"206", "Partial Content"},
-        {"300", "Multiple Choices"},
-        {"305", "Use Proxy"},
-        {"402", "Payment Required"},
-        {"406", "Not Acceptable"},
-        {"407", "Proxy Authentication Required"},
-        {"409", "Conflict"},
-        {"410", "Gone"},
-        {"411", "Length Required"},
-        {"412", "Precondition Failed"},
-        {"413", "Request Entity Too Large"},
-        {"414", "Request-URI Too Long"},
-        {"415", "Unsupported Media Type"},
-        {"416", "Requested Range Not Satisfiable"},
-        {"417", "Expectation Failed"},
-        {"501", "Not Implemented"},
-        {"502", "Bad Gateway"},
-        {"503", "Service Unavailable"},
-        {"504", "Gateway Timeout"},
-        {"505", "HTTP Version Not Supported"},
-        { "", NULL },
-};
-
-
-int psgi_response(struct wsgi_request *wsgi_req, PerlInterpreter *my_perl, AV *response) {
+int psgi_response(struct wsgi_request *wsgi_req, AV *response) {
 
 	SV **status_code, **hitem ;
 	AV *headers, *body =NULL;
@@ -171,9 +126,7 @@ int psgi_response(struct wsgi_request *wsgi_req, PerlInterpreter *my_perl, AV *r
 	vi = (i*2)+base;
         wsgi_req->hvec[vi].iov_base = "\r\n"; wsgi_req->hvec[vi].iov_len = 2;
 
-        if ( !(wsgi_req->headers_size += wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, vi+1)) ) {
-                uwsgi_error("writev()");
-        }
+        wsgi_req->headers_size += wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, vi+1);
 
         hitem = av_fetch(response, 2, 0);
 
@@ -187,14 +140,22 @@ int psgi_response(struct wsgi_request *wsgi_req, PerlInterpreter *my_perl, AV *r
 
 		// respond to fileno ?
 		if (uwsgi.async < 2) {
-			if (uwsgi_perl_obj_can(*hitem, "fileno", 6)) {
+			// check for fileno() method, IO class or GvIO
+			if (uwsgi_perl_obj_can(*hitem, "fileno", 6) || uwsgi_perl_obj_isa(*hitem, "IO") || (uwsgi_perl_obj_isa(*hitem, "GLOB") && GvIO(SvRV(*hitem)))  ) { 
 				SV *fn = uwsgi_perl_obj_call(*hitem, "fileno");
-				wsgi_req->sendfile_fd = SvIV(fn);
-				SvREFCNT_dec(fn);	
-				wsgi_req->response_size += uwsgi_sendfile(wsgi_req);
-				return UWSGI_OK;
+				if (fn) { 
+ 		                                        if (SvTYPE(fn) == SVt_IV && SvIV(fn) >= 0) { 
+ 		                                                wsgi_req->sendfile_fd = SvIV(fn); 
+ 		                                                SvREFCNT_dec(fn);        
+ 		                                                wsgi_req->response_size += uwsgi_sendfile(wsgi_req); 
+ 		                                                // no need to close here as perl GC will do the close() 
+ 		                                                return UWSGI_OK; 
+ 		                                        } 
+ 		                                        SvREFCNT_dec(fn);        
+ 		                                } 
 			}
 			
+			// check for path method 
 			if (uwsgi_perl_obj_can(*hitem, "path", 4)) {
 				SV *p = uwsgi_perl_obj_call(*hitem, "path");
 				wsgi_req->sendfile_fd = open(SvPV_nolen(p), O_RDONLY);
