@@ -76,13 +76,21 @@ XS(XS_wait_fd_write) {
 
 #endif
 
+XS(XS_signal) {
+	dXSARGS;
+
+	psgi_check_args(1);
+
+	uwsgi_signal_send(uwsgi.signal_socket, SvIV(ST(0)));
+
+	XSRETURN_UNDEF;
+}
 
 XS(XS_reload) {
     dXSARGS;
 
-	psgi_check_args(0);
+    psgi_check_args(0);
 
-    uwsgi_log("SENDING HUP TO %d\n", (int) uwsgi.workers[0].pid);
     if (kill(uwsgi.workers[0].pid, SIGHUP)) {
     	uwsgi_error("kill()");
         XSRETURN_NO;
@@ -96,14 +104,19 @@ XS(XS_cache_set) {
 	char *key, *val;
 	STRLEN keylen;
 	STRLEN vallen;
+
+	if (uwsgi.cache_max_items == 0) goto clear;
 	
 	psgi_check_args(2);
 
 	key = SvPV(ST(0), keylen);
 	val = SvPV(ST(1), vallen);
 
+	uwsgi_wlock(uwsgi.cache_lock);
 	uwsgi_cache_set(key, (uint16_t) keylen, val, (uint64_t) vallen, 0, 0);
+	uwsgi_rwunlock(uwsgi.cache_lock);
 
+clear:
 	XSRETURN_UNDEF;
 }
 
@@ -113,17 +126,23 @@ XS(XS_cache_get) {
 	char *key, *val;
 	STRLEN keylen;
 	uint64_t vallen;
+
+	if (uwsgi.cache_max_items == 0) goto clear;
 	
 	psgi_check_args(1);
 
 	key = SvPV(ST(0), keylen);
 
+	uwsgi_rlock(uwsgi.cache_lock);
 	val = uwsgi_cache_get(key, (uint16_t) keylen, &vallen);
 
 	if (!val)
+		uwsgi_rwunlock(uwsgi.cache_lock);
+clear:
 		XSRETURN_UNDEF;
 
 	ST(0) = newSVpv(val, vallen);
+	uwsgi_rwunlock(uwsgi.cache_lock);
 	sv_2mortal(ST(0));
 	
 	XSRETURN(1);
@@ -155,26 +174,28 @@ XS(XS_call) {
 
 	dXSARGS;
 
-	char buffer[0xffff];
         char *func;
         uint16_t size = 0;
         int i;
-        char *argv[0xff];
+        char *argv[256];
+        uint16_t argvs[256];
+	STRLEN arg_len;
 
 	psgi_check_args(1);
 
         func = SvPV_nolen(ST(0));
 
         for(i=0;i<(items-1);i++) {
-                argv[i] = SvPV_nolen(ST(i+1));
+                argv[i] = SvPV(ST(i+1), arg_len);
+		argvs[i] = arg_len;
         }
 
-        size = uwsgi_rpc(func, items-1, argv, buffer);
+        char *response = uwsgi_do_rpc(NULL, func, items-1, argv, argvs, &size);
 
         if (size > 0) {
-		ST(0) = newSVpv(buffer, size);
+		ST(0) = newSVpv(response, size);
         	sv_2mortal(ST(0));
-
+		free(response);
         	XSRETURN(1);
         }
 
@@ -207,5 +228,6 @@ void init_perl_embedded_module() {
 	psgi_xs(log);
 	psgi_xs(async_connect);
 	psgi_xs(suspend);
+	psgi_xs(signal);
 }
 
