@@ -1025,6 +1025,17 @@ PyObject *py_uwsgi_setprocname(PyObject * self, PyObject * args) {
 	return Py_None;
 }
 
+PyObject *py_uwsgi_ready(PyObject * self, PyObject * args) {
+
+	if (ushared->ready) {
+		Py_INCREF(Py_True);
+		return Py_True;
+	}
+
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
 PyObject *py_uwsgi_in_farm(PyObject * self, PyObject * args) {
 
 	char *farm_name = NULL;
@@ -3103,6 +3114,8 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 	{"mule_get_msg", (PyCFunction) py_uwsgi_mule_get_msg, METH_VARARGS|METH_KEYWORDS, ""},
 	{"farm_get_msg", py_uwsgi_farm_get_msg, METH_VARARGS, ""},
 	{"in_farm", py_uwsgi_in_farm, METH_VARARGS, ""},
+
+	{"ready", py_uwsgi_ready, METH_VARARGS, ""},
 	//{"call_hook", py_uwsgi_call_hook, METH_VARARGS, ""},
 
 	{NULL, NULL},
@@ -3138,7 +3151,7 @@ PyObject *py_uwsgi_cache_del(PyObject * self, PyObject * args) {
 	else if (uwsgi.cache_max_items) {
 		UWSGI_RELEASE_GIL
 		uwsgi_wlock(uwsgi.cache_lock);
-		if (uwsgi_cache_del(key, keylen)) {
+		if (uwsgi_cache_del(key, keylen, 0)) {
 			uwsgi_rwunlock(uwsgi.cache_lock);
 			UWSGI_GET_GIL
 			Py_INCREF(Py_None);
@@ -3253,15 +3266,25 @@ PyObject *py_uwsgi_cache_exists(PyObject * self, PyObject * args) {
 	
 	if (remote && strlen(remote) > 0) {
 		// TODO FIX THIS !!!
+		UWSGI_RELEASE_GIL
 		uwsgi_simple_message_string(remote, 111, 0, key, keylen, buffer, &valsize, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT]);
+		UWSGI_GET_GIL
 		if (valsize > 0) {
 			Py_INCREF(Py_True);
 			return Py_True;
 		}	
         }
-	else if (uwsgi_cache_exists(key, keylen)) {
-		Py_INCREF(Py_True);
-		return Py_True;
+	else if (uwsgi.cache_max_items) {
+		UWSGI_RELEASE_GIL
+		uwsgi_rlock(uwsgi.cache_lock);
+		if (uwsgi_cache_exists(key, keylen)) {
+			uwsgi_rwunlock(uwsgi.cache_lock);
+			UWSGI_GET_GIL
+			Py_INCREF(Py_True);
+			return Py_True;
+		}
+		uwsgi_rwunlock(uwsgi.cache_lock);
+		UWSGI_GET_GIL
 	}
 
 	Py_INCREF(Py_None);
@@ -3353,29 +3376,29 @@ PyObject *py_uwsgi_queue_pull(PyObject * self, PyObject * args) {
 	char *message;
 	uint64_t size;
 	PyObject *res;
+	char *storage;
 
 	if (uwsgi.queue_size) {
 		UWSGI_RELEASE_GIL
 		uwsgi_wlock(uwsgi.queue_lock);
+
 		message = uwsgi_queue_pull(&size);
-		UWSGI_GET_GIL
-		if (message && size > 0) {
-			res = PyString_FromStringAndSize(NULL, size);
-#ifdef PYTHREE
-                	char *storage = PyBytes_AsString(res);
-#else
-                	char *storage = PyString_AS_STRING(res);
-#endif
-			UWSGI_RELEASE_GIL
-			memcpy(storage, message, size);
-                }
-                else {
+
+                if (!message || size == 0) {
+                	uwsgi_rwunlock(uwsgi.queue_lock);
+			UWSGI_GET_GIL
                         Py_INCREF(Py_None);
-                        res = Py_None;
-			UWSGI_RELEASE_GIL
+                        return Py_None;
                 }
+
+                storage = uwsgi_malloc(size);
+		memcpy(storage, message, size);
+
                 uwsgi_rwunlock(uwsgi.queue_lock);
 		UWSGI_GET_GIL
+
+		res = PyString_FromStringAndSize(storage, size);
+		free(storage);
                 return res;
 	}
 
@@ -3389,29 +3412,29 @@ PyObject *py_uwsgi_queue_pop(PyObject * self, PyObject * args) {
         char *message;
         uint64_t size;
         PyObject *res;
+	char *storage;
 
         if (uwsgi.queue_size) {
+
 		UWSGI_RELEASE_GIL
                 uwsgi_wlock(uwsgi.queue_lock);
+
                 message = uwsgi_queue_pop(&size);
-		UWSGI_GET_GIL
-		if (message && size > 0) {
-                        res = PyString_FromStringAndSize(NULL, size);
-#ifdef PYTHREE
-                        char *storage = PyBytes_AsString(res);
-#else
-                        char *storage = PyString_AS_STRING(res);
-#endif
-                        UWSGI_RELEASE_GIL
-                        memcpy(storage, message, size);
-                }
-                else {
+		if (!message || size == 0) {
+                	uwsgi_rwunlock(uwsgi.queue_lock);
+			UWSGI_GET_GIL
                         Py_INCREF(Py_None);
-                        res = Py_None;
-                        UWSGI_RELEASE_GIL
-                }
+			return Py_None;
+		}
+
+		storage = uwsgi_malloc(size);
+                memcpy(storage, message, size);
+
                 uwsgi_rwunlock(uwsgi.queue_lock);
 		UWSGI_GET_GIL
+
+		res = PyString_FromStringAndSize(storage, size);
+		free(storage);
                 return res;
         }
 
@@ -3427,6 +3450,7 @@ PyObject *py_uwsgi_queue_get(PyObject * self, PyObject * args) {
 	uint64_t size = 0;
 	char *message;
 	PyObject *res;
+	char *storage;
 
 	if (!PyArg_ParseTuple(args, "l:queue_get", &index)) {
                 return NULL;
@@ -3435,25 +3459,23 @@ PyObject *py_uwsgi_queue_get(PyObject * self, PyObject * args) {
 	if (uwsgi.queue_size) {
 		UWSGI_RELEASE_GIL
 		uwsgi_rlock(uwsgi.queue_lock);
+
 		message = uwsgi_queue_get(index, &size);
-		UWSGI_GET_GIL
-                if (message && size > 0) {
-                        res = PyString_FromStringAndSize(NULL, size);
-#ifdef PYTHREE  
-                        char *storage = PyBytes_AsString(res);
-#else
-                        char *storage = PyString_AS_STRING(res);
-#endif  
-                        UWSGI_RELEASE_GIL
-                        memcpy(storage, message, size);
-                }
-                else {
+                if (!message || size == 0) {
+			uwsgi_rwunlock(uwsgi.queue_lock);
+			UWSGI_GET_GIL
                         Py_INCREF(Py_None);
-                        res = Py_None;
-                        UWSGI_RELEASE_GIL
-                }
+			return Py_None;
+		}
+
+		storage = uwsgi_malloc(size);
+                memcpy(storage, message, size);
+
 		uwsgi_rwunlock(uwsgi.queue_lock);
 		UWSGI_GET_GIL
+
+                res = PyString_FromStringAndSize(storage, size);
+		free(storage);
 		return res;
 	}	
 
@@ -3463,11 +3485,12 @@ PyObject *py_uwsgi_queue_get(PyObject * self, PyObject * args) {
 
 PyObject *py_uwsgi_queue_last(PyObject * self, PyObject * args) {
 
-        long num = 0;
+        long i, num = 0;
         uint64_t size = 0;
         char *message;
         PyObject *res = NULL;
 	uint64_t base;
+	char *storage;
 
         if (!PyArg_ParseTuple(args, "|l:queue_last", &num)) {
                 return NULL;
@@ -3491,50 +3514,41 @@ PyObject *py_uwsgi_queue_last(PyObject * self, PyObject * args) {
 
 		if (num == 0) {
                 	message = uwsgi_queue_get(base, &size);
-			UWSGI_GET_GIL
-                	if (message && size > 0) {
-                        	res = PyString_FromStringAndSize(NULL, size);
-#ifdef PYTHREE  
-                        	char *storage = PyBytes_AsString(res);
-#else
-                        	char *storage = PyString_AS_STRING(res);
-#endif
-                        	UWSGI_RELEASE_GIL
-                        	memcpy(storage, message, size);
-                	}
-                	else {
+                	if (!message || size == 0) {
+                		uwsgi_rwunlock(uwsgi.queue_lock);
+				UWSGI_GET_GIL
                         	Py_INCREF(Py_None);
-                        	res = Py_None;
-                        	UWSGI_RELEASE_GIL
-                	}
+				return Py_None;
+			}
+
+			storage = uwsgi_malloc(size);
+                        memcpy(storage, message, size);
+
                 	uwsgi_rwunlock(uwsgi.queue_lock);
 			UWSGI_GET_GIL
+
+			res = PyString_FromStringAndSize(storage, size);
+			free(storage);
 			return res;
 		}
 
 		if (num > (long)uwsgi.queue_size) num = uwsgi.queue_size;
 
+		char **queue_items = uwsgi_malloc(sizeof(char *) * num);
+		uint64_t *queue_items_size = uwsgi_malloc(sizeof(uint64_t) * num);
+		long item_pos = 0;
 		while(num) {
                 	message = uwsgi_queue_get(base, &size);
-			UWSGI_GET_GIL
-                	if (message && size) {
-				PyObject *zero = PyString_FromStringAndSize(NULL, size);
-				PyList_Append(res, zero);
-				Py_DECREF(zero);
-#ifdef PYTHREE  
-                                char *storage = PyBytes_AsString(res);
-#else           
-                                char *storage = PyString_AS_STRING(res);
-#endif
-				UWSGI_RELEASE_GIL
-				memcpy(storage, message, size);
-                	}
-                	else {
-				UWSGI_RELEASE_GIL
-                		uwsgi_rwunlock(uwsgi.queue_lock);
-				UWSGI_GET_GIL
-				return res;
+                	if (!message || size == 0) {
+				queue_items[item_pos] = NULL;
+				queue_items_size[item_pos] = 0;
                 	}
+			else {
+				queue_items[item_pos] = uwsgi_malloc(size);
+				memcpy(queue_items[item_pos], message, size);
+				queue_items_size[item_pos] = size;
+			}
+			item_pos++;
 			if (base > 0) {
 				base--;
 			}
@@ -3543,9 +3557,24 @@ PyObject *py_uwsgi_queue_last(PyObject * self, PyObject * args) {
 			}
 			num--;
 		}
-		UWSGI_RELEASE_GIL
+
                 uwsgi_rwunlock(uwsgi.queue_lock);
 		UWSGI_GET_GIL
+
+		for(i=0;i<item_pos;i++) {
+			if (queue_items[i]) {
+				PyObject *zero = PyString_FromStringAndSize(queue_items[i], queue_items_size[i]);
+				PyList_Append(res, zero);
+				Py_DECREF(zero);
+				free(queue_items[i]);
+			}
+			else {
+				Py_INCREF(Py_None);
+				PyList_Append(res, Py_None);
+			}
+		}
+		free(queue_items);
+		free(queue_items_size);
                 return res;
         }
 
@@ -3595,14 +3624,7 @@ PyObject *py_uwsgi_cache_get(PyObject * self, PyObject * args) {
 			Py_INCREF(Py_None);
 			return Py_None;
 		}
-		UWSGI_GET_GIL
-		ret = PyString_FromStringAndSize(NULL, valsize);
-#ifdef PYTHREE
-		char *storage = PyBytes_AsString(ret);
-#else
-		char *storage = PyString_AS_STRING(ret);
-#endif
-		UWSGI_RELEASE_GIL
+		char *storage = uwsgi_malloc(valsize);
 #ifdef UWSGI_DEBUG
 		gettimeofday(&tv2, NULL); 
 		if ((tv2.tv_sec* (1000*1000) + tv2.tv_usec) - (tv.tv_sec* (1000*1000) + tv.tv_usec) > 30000) {
@@ -3612,6 +3634,8 @@ PyObject *py_uwsgi_cache_get(PyObject * self, PyObject * args) {
 		memcpy(storage, value, valsize);
 		uwsgi_rwunlock(uwsgi.cache_lock);
 		UWSGI_GET_GIL
+		ret = PyString_FromStringAndSize(storage, valsize);
+		free(storage);
 		return ret;
 	}
 
