@@ -134,7 +134,6 @@ void *uwsgi_request_subhandler_web3(struct wsgi_request *wsgi_req, struct uwsgi_
 int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 
 	PyObject *pychunk;
-	ssize_t wsize;
 
 	// ok its a yield
 	if (!wsgi_req->async_placeholder) {
@@ -159,20 +158,28 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 			Py_INCREF(headers);
 			PyTuple_SetItem(spit_args, 1, headers);
 
-			if (py_uwsgi_spit(NULL, spit_args) == Py_None) { 
+			if (py_uwsgi_spit(Py_None, spit_args) == NULL) { 
+				PyErr_Print();
 				Py_DECREF(spit_args);
 				goto clear; 
 			} 
 
+			// send the headers if not already sent
+        		if (!wsgi_req->headers_sent && wsgi_req->headers_hvec > 0) {
+                		uwsgi_python_do_send_headers(wsgi_req);
+        		}
 
 			Py_DECREF(spit_args);
 
 			if (PyString_Check((PyObject *)wsgi_req->async_placeholder)) {
-                		if ((wsize = wsgi_req->socket->proto_write(wsgi_req, PyString_AsString(wsgi_req->async_placeholder), PyString_Size(wsgi_req->async_placeholder))) < 0) {
-                        		uwsgi_error("write()");
-                        		goto clear;
+				char *content = PyString_AsString(wsgi_req->async_placeholder);
+				size_t content_len = PyString_Size(wsgi_req->async_placeholder);
+				UWSGI_RELEASE_GIL
+                		wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, content, content_len);
+				UWSGI_GET_GIL
+				uwsgi_py_check_write_errors {
+                        		uwsgi_py_write_exception(wsgi_req);
                 		}
-                		wsgi_req->response_size += wsize;
                 		goto clear;
         		}
 
@@ -208,12 +215,16 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 
 
 	if (PyString_Check(pychunk)) {
-		if ((wsize = wsgi_req->socket->proto_write(wsgi_req,  PyString_AsString(pychunk), PyString_Size(pychunk))) < 0) {
-			uwsgi_error("write()");
+		char *content = PyString_AsString(pychunk);
+		size_t content_len = PyString_Size(pychunk);
+		UWSGI_RELEASE_GIL
+		wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, content, content_len);
+		UWSGI_GET_GIL
+		uwsgi_py_check_write_errors {
+			uwsgi_py_write_exception(wsgi_req);
 			Py_DECREF(pychunk);
 			goto clear;
 		}
-		wsgi_req->response_size += wsize;
 	}
 
 
