@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.1.2'
+uwsgi_version = '1.2.3'
 
 import os
 import re
@@ -126,6 +126,8 @@ def build_uwsgi(uc, print_only=False):
         epc = "-DUWSGI_DECLARE_EMBEDDED_PLUGINS=\""
         eplc = "-DUWSGI_LOAD_EMBEDDED_PLUGINS=\""
         for p in ep:
+            if p is None or p == 'None':
+                continue
             p = p.strip()
             if p == 'ugreen':
                 if uwsgi_os == 'OpenBSD' or uwsgi_cpu[0:3] == 'arm' or uwsgi_os == 'Haiku':
@@ -158,6 +160,8 @@ def build_uwsgi(uc, print_only=False):
         if len(ep) > 0:
             print("*** uWSGI compiling embedded plugins ***")
             for p in ep:
+                if p is None or p == 'None':
+                    continue
                 p = p.strip()
 
                 if p == 'ugreen':
@@ -220,7 +224,7 @@ def build_uwsgi(uc, print_only=False):
                 print("*** building plugin: %s ***" % p)
                 build_plugin("plugins/%s" % p, uc, cflags, ldflags, libs)
 
-    bin_name = uc.get('bin_name')
+    bin_name = os.environ.get('UWSGI_BIN_NAME', uc.get('bin_name'))
 
     if uc.get('embed_config'):
         gcc_list.append(uc.get('embed_config'))
@@ -306,6 +310,11 @@ class uConf(object):
                 raise 
         except:
             self.include_path = ['/usr/include', '/usr/local/include']
+
+        additional_include_paths = self.get('additional_include_paths')
+        if additional_include_paths:
+            for ipath in additional_include_paths.split():
+                self.include_path.append(ipath)
             
         if not mute:
             print("detected include path: %s" % self.include_path)
@@ -335,7 +344,10 @@ class uConf(object):
             if not inherit.endswith('.ini'):
                 inherit = '%s.ini' % inherit
 
-            iconfig = ConfigParser.ConfigParser()
+            interpolations = {}
+            for option in self.config.options('uwsgi'):
+                interpolations[option] = self.get(option)
+            iconfig = ConfigParser.ConfigParser(interpolations)
             iconfig.read(inherit)
             for opt in iconfig.options('uwsgi'):
                 if not self.config.has_option('uwsgi', opt):
@@ -383,6 +395,23 @@ class uConf(object):
         if self.has_include('ifaddrs.h'):
             self.cflags.append('-DUWSGI_HAS_IFADDRS')
 
+        if uwsgi_os in ('FreeBSD', 'OpenBSD'):
+            if self.has_include('execinfo.h') or os.path.exists('/usr/local/include/execinfo.h'):
+                if os.path.exists('/usr/local/include/execinfo.h'):
+                    self.cflags.append('-I/usr/local/include')
+                    self.ldflags.append('-L/usr/local/lib')
+                self.cflags.append('-DUWSGI_HAS_EXECINFO')
+                self.libs.append('-lexecinfo')
+
+        if uwsgi_os == 'OpenBSD':
+            try:
+                obsd_major = int(uwsgi_os_k.split('.')[0])
+                obsd_minor = int(uwsgi_os_k.split('.')[1])
+                if obsd_major >= 5 and obsd_minor > 0:
+                    self.cflags.append('-DUWSGI_NEW_OPENBSD')
+            except:
+                pass
+
         if uwsgi_os == 'SunOS':
             self.libs.append('-lsendfile')
             self.gcc_list.append('lib/sun_fixes')
@@ -415,16 +444,22 @@ class uConf(object):
             if uwsgi_os == 'Linux' or uwsgi_os == 'SunOS':
                 locking_mode = 'pthread_mutex'
             # FreeBSD umtx is still not ready for process shared locking
-            #elif uwsgi_os == 'FreeBSD':
-            #    locking_mode = 'umtx'
+            # starting from FreeBSD 9 posix semaphores can be shared between processes
+            elif uwsgi_os == 'FreeBSD':
+                 try:
+                     fbsd_major = int(uwsgi_os_k.split('.')[0])
+                     if fbsd_major >= 9:
+                         locking_mode = 'posix_sem'
+                 except:
+                     pass
             elif uwsgi_os == 'Darwin':
                 locking_mode = 'osx_spinlock'
 
         if locking_mode == 'pthread_mutex':
             self.cflags.append('-DUWSGI_LOCK_USE_MUTEX')
         # FreeBSD umtx is still not ready for process shared locking
-        #elif locking_mode == 'umtx':
-        #    self.cflags.append('-DUWSGI_LOCK_USE_UMTX')
+        elif locking_mode == 'posix_sem':
+            self.cflags.append('-DUWSGI_LOCK_USE_POSIX_SEM')
         elif locking_mode == 'osx_spinlock':
             self.cflags.append('-DUWSGI_LOCK_USE_OSX_SPINLOCK')
 
