@@ -3,7 +3,10 @@ import os
 import gc
 import sys
 from uwsgidecorators import *
-gc.set_debug(gc.DEBUG_SAVEALL)
+print(sys.version)
+print(sys.version_info)
+if 'set_debug' in gc.__dict__:
+    gc.set_debug(gc.DEBUG_SAVEALL)
 
 print(os.environ)
 print(sys.modules)
@@ -39,7 +42,8 @@ def xsendfile(e, sr):
     return ''
 
 def serve_logo(e, sr):
-    sr('200 OK', [('Content-Type', 'image/png')])
+    # use raw facilities
+    uwsgi.send("%s 200 OK\r\nContent-Type: image/png\r\n\r\n" % e['SERVER_PROTOCOL'])
     return uwsgi.sendfile('logo_uWSGI.png')
 
 def serve_options(e, sr):
@@ -64,7 +68,6 @@ def setprocname():
         uwsgi.setprocname("i am the worker %d" % uwsgi.worker_id())
 
 def application(env, start_response):
-
     try:
         uwsgi.mule_msg(env['REQUEST_URI'], 1)
     except:
